this.build_Bowyer_event <- this.inherit("scripts/events/event", {
	m = {
		Bowyer = null,
		Town = null,
		Costs = 5000,
	},
	function create()
	{
		this.m.ID = "event.build_Bowyer";
		this.m.Title = "At %townname%";
		this.m.Cooldown = 1.0 * this.World.getTime().SecondsPerDay;
		this.m.Screens.push({
			ID = "A",
			Text = "[img]gfx/ui/events/event_20.png[/img]%bowyer% the bowyer comes to you with a bit of request: he wants build here a fletcher and stop being sellsword. Anyway he is not a really archer he wants to make a bows as before. He tells that it was too many deaths and he can\'t sleep at night - he suffers by nightmares. But he has a big problem - money, building fletcher is very expensive, about 5.000 crowns. May be company can help him with this.",
			Image = "",
			List = [],
			Characters = [],
			Options = [
				{
					Text = "I will give you all 5.000 crowns just now.",
					function getResult( _event )
					{
						return "B";
					}
				},
				{
					Text = "Company still need you, stay with us.",
					function getResult( _event )
					{
						return 0;
					}
				}
			],
			function start( _event )
			{
				this.Characters.push(_event.m.Bowyer.getImagePath());
			}
		});
		this.m.Screens.push({
			ID = "B",
			Text = "[img]gfx/ui/events/event_20.png[/img]Few minutes late all company bids farewell with %Bowyer%. That is really encouraging moment, knowing that at least one of us has everything what he wants. New life, family, home and job which helps people without smashing heads.",
			Image = "",
			List = [],
			Characters = [],
			Options = [
				{
					Text = "Don\'t forget us, we are your family forever.",
					function getResult( _event )
					{
						return 0;
					}
				}
			],
			function start( _event )
			{
				this.Characters.push(_event.m.Bowyer.getImagePath());

				this.World.Assets.addMoney(-_event.m.Costs);
				_event.m.Town.addBuilding(this.new("scripts/entity/world/settlements/buildings/fletcher_building"));

				_event.m.Bowyer.getItems().transferToStash(this.World.Assets.getStash());
				this.World.getPlayerRoster().remove(_event.m.Bowyer);
				_event.m.Bowyer = null;

				this.List.push({
					id = 10,
					icon = "ui/icons/asset_money.png",
					text = "You spend [color=" + this.Const.UI.Color.NegativeEventValue + "]" + _event.m.Costs + "[/color] Crowns"
				});

				this.List.push({
					id = 10,
					icon = "ui/icons/special.png",
					text = _event.m.Bowyer.getName() + " left company"
				});

				this.List.push({
					id = 10,
					icon = "ui/icons/buildings/fletcher.png",
					text = _event.m.Town.getName() + " now has fletcher"
				});

				this.World.FactionManager.getFaction(_event.m.Town.getFactions()[0]).addPlayerRelation(this.Const.World.Assets.ReputationOnAmbition, "You built fletcher");
			}
		});
	}

	function onUpdateScore()
	{
		if (this.World.Assets.getMoney() < this.m.Costs)
		{
			return;
		}

		local towns = this.World.EntityManager.getSettlements();
		local nearTown = false;
		local town;
		local playerTile = this.World.State.getPlayer().getTile();

		foreach( t in towns )
		{
			if (t.getTile().getDistanceTo(playerTile) > 3 || !t.isAlliedWithPlayer())
			{
				continue;
			}

			if (t.getSize() > 1 || t.isMilitary())
			{
				continue;
			}

			if (t.hasBuilding("building.temple") || t.hasBuilding("building.armorsmith") || t.hasBuilding("building.weaponsmith") || t.hasBuilding("building.fletcher"))
			{
				continue;
			}

			nearTown = true;
			town = t;
			break;
		}

		if (!nearTown)
		{
			return;
		}

		local brothers = this.World.getPlayerRoster().getAll();
		local candidates = [];

		foreach( bro in brothers )
		{
			if (bro.getBackground().getID() == "background.bowyer")
			{
				candidates.push(bro);
			}
		}

		if (candidates.len() == 0)
		{
			return;
		}

		this.m.Bowyer = candidates[this.Math.rand(0, candidates.len() - 1)];
		this.m.Town = town;
		this.m.Score = 500;
	}

	function onPrepare()
	{
	}

	function onPrepareVariables( _vars )
	{
		_vars.push([
			"bowyer",
			this.m.Bowyer.getNameOnly()
		]);
		_vars.push([
			"townname",
			this.m.Town.getName()
		]);
	}

	function onClear()
	{
		this.m.Bowyer = null;
		this.m.Town = null;
	}
});
