this.build_temple_event <- this.inherit("scripts/events/event", {
	m = {
		Monk = null,
		Town = null,
		Costs = 5000,
	},
	function create()
	{
		this.m.ID = "event.build_temple";
		this.m.Title = "At %townname%";
		this.m.Cooldown = 1.0 * this.World.getTime().SecondsPerDay;
		this.m.Screens.push({
			ID = "A",
			Text = "[img]gfx/ui/events/event_20.png[/img]%monk% the monk comes to you and asks can he leaves company and build temple here - God told him this wish. It is very strange but all religious people sometimes hear the voice. Anyway to build temple here is a big deal, expensive at least. If only company can give 5.000 - it will be generous. Do you believe in the God as this monk?",
			Image = "",
			List = [],
			Characters = [],
			Options = [
				{
					Text = "I will give you all 5.000 crowns just now.",
					function getResult( _event )
					{
						return "B";
					}
				},
				{
					Text = "Company still need you, stay with us.",
					function getResult( _event )
					{
						return 0;
					}
				}
			],
			function start( _event )
			{
				this.Characters.push(_event.m.Monk.getImagePath());
			}
		});
		this.m.Screens.push({
			ID = "B",
			Text = "[img]gfx/ui/events/event_20.png[/img]Few minutes late all company bids farewell with %monk%. That is really encouraging moment, knowing that at least one of us has everything what he wants. %monk% tells that he as praying for us, souls and lifes.",
			Image = "",
			List = [],
			Characters = [],
			Options = [
				{
					Text = "Don\'t forget us, we are your family forever.",
					function getResult( _event )
					{
						return 0;
					}
				}
			],
			function start( _event )
			{
				this.Characters.push(_event.m.Monk.getImagePath());

				this.World.Assets.addMoney(-_event.m.Costs);
				_event.m.Town.addBuilding(this.new("scripts/entity/world/settlements/buildings/temple_building"));

				_event.m.Monk.getItems().transferToStash(this.World.Assets.getStash());
				this.World.getPlayerRoster().remove(_event.m.Monk);
				_event.m.Monk = null;

				this.List.push({
					id = 10,
					icon = "ui/icons/asset_money.png",
					text = "You spend [color=" + this.Const.UI.Color.NegativeEventValue + "]" + _event.m.Costs + "[/color] Crowns"
				});

				this.List.push({
					id = 10,
					icon = "ui/icons/special.png",
					text = _event.m.Monk.getName() + " left company"
				});

				this.List.push({
					id = 10,
					icon = "ui/icons/buildings/temple.png",
					text = _event.m.Town.getName() + " now has temple"
				});

				this.World.FactionManager.getFaction(_event.m.Town.getFactions()[0]).addPlayerRelation(this.Const.World.Assets.ReputationOnAmbition, "You built temple");
			}
		});
	}

	function onUpdateScore()
	{
		if (this.World.Assets.getMoney() < this.m.Costs)
		{
			return;
		}

		local towns = this.World.EntityManager.getSettlements();
		local nearTown = false;
		local town;
		local playerTile = this.World.State.getPlayer().getTile();

		foreach( t in towns )
		{
			if (t.getTile().getDistanceTo(playerTile) > 3 || !t.isAlliedWithPlayer())
			{
				continue;
			}

			if (t.getSize() > 1 || t.isMilitary())
			{
				continue;
			}

			if (t.hasBuilding("building.temple") || t.hasBuilding("building.armorsmith") || t.hasBuilding("building.weaponsmith") || t.hasBuilding("building.fletcher"))
			{
				continue;
			}

			nearTown = true;
			town = t;
			break;
		}

		if (!nearTown)
		{
			return;
		}

		local brothers = this.World.getPlayerRoster().getAll();
		local candidates = [];

		foreach( bro in brothers )
		{
			if (bro.getBackground().getID() == "background.monk")
			{
				candidates.push(bro);
			}
		}

		if (candidates.len() == 0)
		{
			return;
		}

		this.m.Monk = candidates[this.Math.rand(0, candidates.len() - 1)];
		this.m.Town = town;
		this.m.Score = 500;
	}

	function onPrepare()
	{
	}

	function onPrepareVariables( _vars )
	{
		_vars.push([
			"monk",
			this.m.Monk.getNameOnly()
		]);
		_vars.push([
			"townname",
			this.m.Town.getName()
		]);
	}

	function onClear()
	{
		this.m.Monk = null;
		this.m.Town = null;
	}
});
